package com.persagy.apm.energy.report.config.itemgroup.model;

import org.mapstruct.*;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;
import com.persagy.apm.energy.report.config.itemgroup.model.vo.*;
import com.persagy.apm.energy.report.config.itemgroup.model.dto.*;

import java.util.List;

/**
 * 分项分组(ItemGroup) dto、vo、do转换工具类
 *
 * @author lixing
 * @version V1.0 2021-05-19 11:17:49 2021-05-19 11:17:49
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertItemGroupTool {

    ConvertItemGroupTool INSTANCE = Mappers.getMapper(ConvertItemGroupTool.class);

    /**
     * do转换为ResponseItemVO
     *
     * @param itemGroup do对象
     * @return ResponseItemVO
     * @author lixing
     * @version V1.0 2021-05-19 11:17:49
     */
    ResponseItemGroupItemVO convert2ResponseItemDTO(ItemGroup itemGroup);

    /**
     * do转换为ResponseListItemVO
     *
     * @param itemGroup do对象
     * @return ResponseListItemVO
     * @author lixing
     * @version V1.0 2021-05-19 11:17:49
     */
    ResponseItemGroupListItemVO convert2ResponseListItemDTO(ItemGroup itemGroup);

    /**
     * do列表转换为ResponseListItemVO列表
     *
     * @param itemGroupList do列表
     * @return ResponseListItemVO列表
     * @author lixing
     * @version V1.0 2021-05-19 11:17:49
     */
    List<ResponseItemGroupListItemVO> convert2List(List<ItemGroup> itemGroupList);

    /**
     * addDTO转换为do
     *
     * @param addItemGroupDTO addDTO
     * @return 要创建的do对象
     * @author lixing
     * @version V1.0 2021-05-19 11:17:49
     */
    ItemGroup convertAddDto2Entity(AddItemGroupDTO addItemGroupDTO);

    /**
     * updateDTO转换为实体
     *
     * @param itemGroup          更新前的do对象
     * @param updateItemGroupDTO updateDTO
     * @return 更新后的do对象
     * @author lixing
     * @version V1.0 2021-05-19 11:17:49
     */
    ItemGroup convertUpdateDto2Entity(@MappingTarget ItemGroup itemGroup, UpdateItemGroupDTO updateItemGroupDTO);
}

