package com.persagy.apm.energy.report.config.itemgroup.service.impl;

import com.persagy.apm.common.context.poems.PoemsContext;
import com.persagy.apm.energy.report.config.itemgroup.dao.ItemGroupMapper;
import com.persagy.apm.energy.report.config.itemgroup.service.IItemGroupService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.apache.commons.lang.StringUtils;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.energy.report.config.itemgroup.model.*;
import com.persagy.apm.energy.report.config.itemgroup.model.dto.*;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.model.dto.Sort;
import org.springframework.util.CollectionUtils;

/**
 * 分项分组(ItemGroup) service层
 *
 * @author lixing
 * @version V1.0 2021-05-19 11:17:46
 */
@Service
public class ItemGroupServiceImpl extends ServiceImpl<ItemGroupMapper, ItemGroup>
        implements IItemGroupService {

    /**
     * 创建分项分组
     *
     * @return 分项分组主键
     * @author lixing
     * @version V1.0 2021-05-19 11:17:46
     */
    @Override
    public String createItemGroup(AddItemGroupDTO addItemGroupDTO) {
        ItemGroup itemGroup = ConvertItemGroupTool.INSTANCE.convertAddDto2Entity(addItemGroupDTO);
        // 设置默认值
        setDefaultValue(itemGroup);
        save(itemGroup);
        return itemGroup.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param itemGroup 分项分组实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ItemGroup itemGroup) {
        itemGroup.setCreator(PoemsContext.getContext().getUserId());
        // todo 其他默认的属性

    }

    /**
     * 分项分组详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-05-19 11:17:46
     */
    @Override
    public ItemGroup queryItemGroupDetail(String id) {
        ItemGroup itemGroup = getById(id);
        if (itemGroup == null) {
            throw new IllegalArgumentException("查看ItemGroup详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return itemGroup;
    }

    /**
     * 更新分项分组
     *
     * @author lixing
     * @version V1.0 2021-05-19 11:17:46
     */
    @Override
    public void updateItemGroup(UpdateItemGroupDTO updateItemGroupDTO) {
        ItemGroup itemGroup = getById(updateItemGroupDTO.getId());
        itemGroup = ConvertItemGroupTool.INSTANCE.convertUpdateDto2Entity(itemGroup, updateItemGroupDTO);
        itemGroup.setModifier(PoemsContext.getContext().getUserId());
        updateById(itemGroup);
    }

    /**
     * 校验分项分组是否可删除
     *
     * @param id 分项分组主键
     * @return 分项分组do类
     * @author lixing
     * @version V1.0 2021-05-19 11:17:46
     */
    public ItemGroup checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ItemGroup时发生异常，主键为空");
        }

        ItemGroup itemGroup = getById(id);

        if (itemGroup == null) {
            throw new IllegalArgumentException("删除ItemGroup时发生异常，找不到要删除的数据，id:" + id);
        }

        return itemGroup;
    }

    /**
     * 删除分项分组
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-05-19 11:17:46
     */
    @Override
    public void deleteItemGroup(String id) {
        // 校验是否可删除
        ItemGroup itemGroup = checkDeletable(id);

        itemGroup.setValid(ValidEnum.FALSE.getType());
        updateById(itemGroup);
    }

    /**
     * 查询分项分组
     *
     * @return List<ItemGroup>
     * @author lixing
     * @version V1.0 2021-05-19 11:17:46
     */
    @Override
    public List<ItemGroup> queryItemGroupList(QueryItemGroupDTO queryItemGroupDTO) {
        QueryWrapper<ItemGroup> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ItemGroup.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ItemGroup.PROP_CREATIONTIME);

        if (queryItemGroupDTO != null) {

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryItemGroupDTO.getName())) {
                queryWrapper.like(ItemGroup.PROP_NAME, queryItemGroupDTO.getName());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryItemGroupDTO.getItems())) {
                queryWrapper.like(ItemGroup.PROP_ITEMS, queryItemGroupDTO.getItems());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询分项分组
     *
     * @return IPage<ItemGroup>
     * @author lixing
     * @version V1.0 2021-05-19 11:17:46
     */
    @Override
    public IPage<ItemGroup> pageQueryItemGroup(PageQueryItemGroupDTO pageQueryItemGroupDTO) {
        QueryWrapper<ItemGroup> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ItemGroup.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ItemGroup> pageParam = new Page<>(pageQueryItemGroupDTO.getPage(), pageQueryItemGroupDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryItemGroupDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, ItemGroup.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryItemGroupDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryItemGroupDTO.getName())) {
            queryWrapper.like(ItemGroup.PROP_NAME, pageQueryItemGroupDTO.getName());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryItemGroupDTO.getItems())) {
            queryWrapper.like(ItemGroup.PROP_ITEMS, pageQueryItemGroupDTO.getItems());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }
}
