package com.persagy.apm.energy.report.config.rel.detailmodel.service.impl;

import com.persagy.apm.common.context.poems.PoemsContext;
import com.persagy.apm.energy.report.config.rel.detailmodel.dao.ReportDetailModelRelMapper;
import com.persagy.apm.energy.report.config.rel.detailmodel.service.IReportDetailModelRelService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.apache.commons.lang.StringUtils;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.energy.report.config.rel.detailmodel.model.*;
import com.persagy.apm.energy.report.config.rel.detailmodel.model.dto.*;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.model.dto.Sort;
import org.springframework.util.CollectionUtils;

/**
 * 报告模型关联关系(ReportDetailModelRel) service层
 *
 * @author lixing
 * @version V1.0 2021-05-19 11:19:09
 */
@Service
public class ReportDetailModelRelServiceImpl extends ServiceImpl<ReportDetailModelRelMapper, ReportDetailModelRel>
        implements IReportDetailModelRelService {

    /**
     * 创建报告模型关联关系
     *
     * @return 报告模型关联关系主键
     * @author lixing
     * @version V1.0 2021-05-19 11:19:09
     */
    @Override
    public String createReportDetailModelRel(AddReportDetailModelRelDTO addReportDetailModelRelDTO) {
        ReportDetailModelRel reportDetailModelRel = ConvertReportDetailModelRelTool.INSTANCE.convertAddDto2Entity(addReportDetailModelRelDTO);
        // 设置默认值
        setDefaultValue(reportDetailModelRel);
        save(reportDetailModelRel);
        return reportDetailModelRel.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param reportDetailModelRel 报告模型关联关系实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ReportDetailModelRel reportDetailModelRel) {
        reportDetailModelRel.setCreator(PoemsContext.getContext().getUserId());
        // todo 其他默认的属性

    }

    /**
     * 报告模型关联关系详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-05-19 11:19:09
     */
    @Override
    public ReportDetailModelRel queryReportDetailModelRelDetail(String id) {
        ReportDetailModelRel reportDetailModelRel = getById(id);
        if (reportDetailModelRel == null) {
            throw new IllegalArgumentException("查看ReportDetailModelRel详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return reportDetailModelRel;
    }

    /**
     * 更新报告模型关联关系
     *
     * @author lixing
     * @version V1.0 2021-05-19 11:19:09
     */
    @Override
    public void updateReportDetailModelRel(UpdateReportDetailModelRelDTO updateReportDetailModelRelDTO) {
        ReportDetailModelRel reportDetailModelRel = getById(updateReportDetailModelRelDTO.getId());
        reportDetailModelRel = ConvertReportDetailModelRelTool.INSTANCE.convertUpdateDto2Entity(reportDetailModelRel, updateReportDetailModelRelDTO);
        reportDetailModelRel.setModifier(PoemsContext.getContext().getUserId());
        updateById(reportDetailModelRel);
    }

    /**
     * 校验报告模型关联关系是否可删除
     *
     * @param id 报告模型关联关系主键
     * @return 报告模型关联关系do类
     * @author lixing
     * @version V1.0 2021-05-19 11:19:09
     */
    public ReportDetailModelRel checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ReportDetailModelRel时发生异常，主键为空");
        }

        ReportDetailModelRel reportDetailModelRel = getById(id);

        if (reportDetailModelRel == null) {
            throw new IllegalArgumentException("删除ReportDetailModelRel时发生异常，找不到要删除的数据，id:" + id);
        }

        return reportDetailModelRel;
    }

    /**
     * 删除报告模型关联关系
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-05-19 11:19:09
     */
    @Override
    public void deleteReportDetailModelRel(String id) {
        // 校验是否可删除
        ReportDetailModelRel reportDetailModelRel = checkDeletable(id);

        reportDetailModelRel.setValid(ValidEnum.FALSE.getType());
        updateById(reportDetailModelRel);
    }

    /**
     * 查询报告模型关联关系
     *
     * @return List<ReportDetailModelRel>
     * @author lixing
     * @version V1.0 2021-05-19 11:19:09
     */
    @Override
    public List<ReportDetailModelRel> queryReportDetailModelRelList(QueryReportDetailModelRelDTO queryReportDetailModelRelDTO) {
        QueryWrapper<ReportDetailModelRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportDetailModelRel.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ReportDetailModelRel.PROP_CREATIONTIME);

        if (queryReportDetailModelRelDTO != null) {

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportDetailModelRelDTO.getReportDetailId())) {
                queryWrapper.like(ReportDetailModelRel.PROP_REPORT_DETAIL_ID, queryReportDetailModelRelDTO.getReportDetailId());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportDetailModelRelDTO.getModelCode())) {
                queryWrapper.like(ReportDetailModelRel.PROP_MODEL_CODE, queryReportDetailModelRelDTO.getModelCode());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportDetailModelRelDTO.getReportTypeId())) {
                queryWrapper.like(ReportDetailModelRel.PROP_REPORT_TYPE_ID, queryReportDetailModelRelDTO.getReportTypeId());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportDetailModelRelDTO.getUnqualifiedReason())) {
                queryWrapper.like(ReportDetailModelRel.PROP_UNQUALIFIED_REASON, queryReportDetailModelRelDTO.getUnqualifiedReason());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportDetailModelRelDTO.getRemark())) {
                queryWrapper.like(ReportDetailModelRel.PROP_REMARK, queryReportDetailModelRelDTO.getRemark());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询报告模型关联关系
     *
     * @return IPage<ReportDetailModelRel>
     * @author lixing
     * @version V1.0 2021-05-19 11:19:09
     */
    @Override
    public IPage<ReportDetailModelRel> pageQueryReportDetailModelRel(PageQueryReportDetailModelRelDTO pageQueryReportDetailModelRelDTO) {
        QueryWrapper<ReportDetailModelRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportDetailModelRel.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ReportDetailModelRel> pageParam = new Page<>(pageQueryReportDetailModelRelDTO.getPage(), pageQueryReportDetailModelRelDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryReportDetailModelRelDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, ReportDetailModelRel.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryReportDetailModelRelDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportDetailModelRelDTO.getReportDetailId())) {
            queryWrapper.like(ReportDetailModelRel.PROP_REPORT_DETAIL_ID, pageQueryReportDetailModelRelDTO.getReportDetailId());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportDetailModelRelDTO.getModelCode())) {
            queryWrapper.like(ReportDetailModelRel.PROP_MODEL_CODE, pageQueryReportDetailModelRelDTO.getModelCode());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportDetailModelRelDTO.getReportTypeId())) {
            queryWrapper.like(ReportDetailModelRel.PROP_REPORT_TYPE_ID, pageQueryReportDetailModelRelDTO.getReportTypeId());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportDetailModelRelDTO.getUnqualifiedReason())) {
            queryWrapper.like(ReportDetailModelRel.PROP_UNQUALIFIED_REASON, pageQueryReportDetailModelRelDTO.getUnqualifiedReason());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportDetailModelRelDTO.getRemark())) {
            queryWrapper.like(ReportDetailModelRel.PROP_REMARK, pageQueryReportDetailModelRelDTO.getRemark());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }
}
