package com.persagy.apm.energy.report.config.rel.modelitem.service.impl;

import com.persagy.apm.common.context.poems.PoemsContext;
import com.persagy.apm.energy.report.config.rel.modelitem.dao.ModelItemRelMapper;
import com.persagy.apm.energy.report.config.rel.modelitem.service.IModelItemRelService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.apache.commons.lang.StringUtils;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.energy.report.config.rel.modelitem.model.*;
import com.persagy.apm.energy.report.config.rel.modelitem.model.dto.*;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.model.dto.Sort;
import org.springframework.util.CollectionUtils;

/**
 * 能耗模型与分项关联关系(ModelItemRel) service层
 *
 * @author lixing
 * @version V1.0 2021-05-19 11:16:51
 */
@Service
public class ModelItemRelServiceImpl extends ServiceImpl<ModelItemRelMapper, ModelItemRel>
        implements IModelItemRelService {

    /**
     * 创建能耗模型与分项关联关系
     *
     * @return 能耗模型与分项关联关系主键
     * @author lixing
     * @version V1.0 2021-05-19 11:16:51
     */
    @Override
    public String createModelItemRel(AddModelItemRelDTO addModelItemRelDTO) {
        ModelItemRel modelItemRel = ConvertModelItemRelTool.INSTANCE.convertAddDto2Entity(addModelItemRelDTO);
        // 设置默认值
        setDefaultValue(modelItemRel);
        save(modelItemRel);
        return modelItemRel.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param modelItemRel 能耗模型与分项关联关系实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ModelItemRel modelItemRel) {
        modelItemRel.setCreator(PoemsContext.getContext().getUserId());
        // todo 其他默认的属性

    }

    /**
     * 能耗模型与分项关联关系详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-05-19 11:16:51
     */
    @Override
    public ModelItemRel queryModelItemRelDetail(String id) {
        ModelItemRel modelItemRel = getById(id);
        if (modelItemRel == null) {
            throw new IllegalArgumentException("查看ModelItemRel详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return modelItemRel;
    }

    /**
     * 更新能耗模型与分项关联关系
     *
     * @author lixing
     * @version V1.0 2021-05-19 11:16:51
     */
    @Override
    public void updateModelItemRel(UpdateModelItemRelDTO updateModelItemRelDTO) {
        ModelItemRel modelItemRel = getById(updateModelItemRelDTO.getId());
        modelItemRel = ConvertModelItemRelTool.INSTANCE.convertUpdateDto2Entity(modelItemRel, updateModelItemRelDTO);
        modelItemRel.setModifier(PoemsContext.getContext().getUserId());
        updateById(modelItemRel);
    }

    /**
     * 校验能耗模型与分项关联关系是否可删除
     *
     * @param id 能耗模型与分项关联关系主键
     * @return 能耗模型与分项关联关系do类
     * @author lixing
     * @version V1.0 2021-05-19 11:16:51
     */
    public ModelItemRel checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ModelItemRel时发生异常，主键为空");
        }

        ModelItemRel modelItemRel = getById(id);

        if (modelItemRel == null) {
            throw new IllegalArgumentException("删除ModelItemRel时发生异常，找不到要删除的数据，id:" + id);
        }

        return modelItemRel;
    }

    /**
     * 删除能耗模型与分项关联关系
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-05-19 11:16:51
     */
    @Override
    public void deleteModelItemRel(String id) {
        // 校验是否可删除
        ModelItemRel modelItemRel = checkDeletable(id);

        modelItemRel.setValid(ValidEnum.FALSE.getType());
        updateById(modelItemRel);
    }

    /**
     * 查询能耗模型与分项关联关系
     *
     * @return List<ModelItemRel>
     * @author lixing
     * @version V1.0 2021-05-19 11:16:51
     */
    @Override
    public List<ModelItemRel> queryModelItemRelList(QueryModelItemRelDTO queryModelItemRelDTO) {
        QueryWrapper<ModelItemRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ModelItemRel.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ModelItemRel.PROP_CREATIONTIME);

        if (queryModelItemRelDTO != null) {

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryModelItemRelDTO.getModelCode())) {
                queryWrapper.like(ModelItemRel.PROP_MODEL_CODE, queryModelItemRelDTO.getModelCode());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryModelItemRelDTO.getItemGroupId())) {
                queryWrapper.like(ModelItemRel.PROP_ITEM_GROUP_ID, queryModelItemRelDTO.getItemGroupId());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryModelItemRelDTO.getItemCode())) {
                queryWrapper.like(ModelItemRel.PROP_ITEM_CODE, queryModelItemRelDTO.getItemCode());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询能耗模型与分项关联关系
     *
     * @return IPage<ModelItemRel>
     * @author lixing
     * @version V1.0 2021-05-19 11:16:51
     */
    @Override
    public IPage<ModelItemRel> pageQueryModelItemRel(PageQueryModelItemRelDTO pageQueryModelItemRelDTO) {
        QueryWrapper<ModelItemRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ModelItemRel.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ModelItemRel> pageParam = new Page<>(pageQueryModelItemRelDTO.getPage(), pageQueryModelItemRelDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryModelItemRelDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, ModelItemRel.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryModelItemRelDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryModelItemRelDTO.getModelCode())) {
            queryWrapper.like(ModelItemRel.PROP_MODEL_CODE, pageQueryModelItemRelDTO.getModelCode());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryModelItemRelDTO.getItemGroupId())) {
            queryWrapper.like(ModelItemRel.PROP_ITEM_GROUP_ID, pageQueryModelItemRelDTO.getItemGroupId());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryModelItemRelDTO.getItemCode())) {
            queryWrapper.like(ModelItemRel.PROP_ITEM_CODE, pageQueryModelItemRelDTO.getItemCode());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }
}
