package com.persagy.apm.energy.report.config.rel.typeprojecttype.service.impl;

import com.persagy.apm.common.context.poems.PoemsContext;
import com.persagy.apm.energy.report.config.rel.typeprojecttype.dao.ReportTypeProjectTypeRelMapper;
import com.persagy.apm.energy.report.config.rel.typeprojecttype.service.IReportTypeProjectTypeRelService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.apache.commons.lang.StringUtils;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.energy.report.config.rel.typeprojecttype.model.*;
import com.persagy.apm.energy.report.config.rel.typeprojecttype.model.dto.*;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.model.dto.Sort;
import org.springframework.util.CollectionUtils;

/**
 * 报告类型和项目所属功能类型关联关系(ReportTypeProjectTypeRel) service层
 *
 * @author lixing
 * @version V1.0 2021-05-19 11:20:23
 */
@Service
public class ReportTypeProjectTypeRelServiceImpl extends ServiceImpl<ReportTypeProjectTypeRelMapper, ReportTypeProjectTypeRel>
        implements IReportTypeProjectTypeRelService {

    /**
     * 创建报告类型和项目所属功能类型关联关系
     *
     * @return 报告类型和项目所属功能类型关联关系主键
     * @author lixing
     * @version V1.0 2021-05-19 11:20:23
     */
    @Override
    public String createReportTypeProjectTypeRel(AddReportTypeProjectTypeRelDTO addReportTypeProjectTypeRelDTO) {
        ReportTypeProjectTypeRel reportTypeProjectTypeRel = ConvertReportTypeProjectTypeRelTool.INSTANCE.convertAddDto2Entity(addReportTypeProjectTypeRelDTO);
        // 设置默认值
        setDefaultValue(reportTypeProjectTypeRel);
        save(reportTypeProjectTypeRel);
        return reportTypeProjectTypeRel.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param reportTypeProjectTypeRel 报告类型和项目所属功能类型关联关系实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ReportTypeProjectTypeRel reportTypeProjectTypeRel) {
        reportTypeProjectTypeRel.setCreator(PoemsContext.getContext().getUserId());
        // todo 其他默认的属性

    }

    /**
     * 报告类型和项目所属功能类型关联关系详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-05-19 11:20:23
     */
    @Override
    public ReportTypeProjectTypeRel queryReportTypeProjectTypeRelDetail(String id) {
        ReportTypeProjectTypeRel reportTypeProjectTypeRel = getById(id);
        if (reportTypeProjectTypeRel == null) {
            throw new IllegalArgumentException("查看ReportTypeProjectTypeRel详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return reportTypeProjectTypeRel;
    }

    /**
     * 更新报告类型和项目所属功能类型关联关系
     *
     * @author lixing
     * @version V1.0 2021-05-19 11:20:23
     */
    @Override
    public void updateReportTypeProjectTypeRel(UpdateReportTypeProjectTypeRelDTO updateReportTypeProjectTypeRelDTO) {
        ReportTypeProjectTypeRel reportTypeProjectTypeRel = getById(updateReportTypeProjectTypeRelDTO.getId());
        reportTypeProjectTypeRel = ConvertReportTypeProjectTypeRelTool.INSTANCE.convertUpdateDto2Entity(reportTypeProjectTypeRel, updateReportTypeProjectTypeRelDTO);
        reportTypeProjectTypeRel.setModifier(PoemsContext.getContext().getUserId());
        updateById(reportTypeProjectTypeRel);
    }

    /**
     * 校验报告类型和项目所属功能类型关联关系是否可删除
     *
     * @param id 报告类型和项目所属功能类型关联关系主键
     * @return 报告类型和项目所属功能类型关联关系do类
     * @author lixing
     * @version V1.0 2021-05-19 11:20:23
     */
    public ReportTypeProjectTypeRel checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ReportTypeProjectTypeRel时发生异常，主键为空");
        }

        ReportTypeProjectTypeRel reportTypeProjectTypeRel = getById(id);

        if (reportTypeProjectTypeRel == null) {
            throw new IllegalArgumentException("删除ReportTypeProjectTypeRel时发生异常，找不到要删除的数据，id:" + id);
        }

        return reportTypeProjectTypeRel;
    }

    /**
     * 删除报告类型和项目所属功能类型关联关系
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-05-19 11:20:23
     */
    @Override
    public void deleteReportTypeProjectTypeRel(String id) {
        // 校验是否可删除
        ReportTypeProjectTypeRel reportTypeProjectTypeRel = checkDeletable(id);

        reportTypeProjectTypeRel.setValid(ValidEnum.FALSE.getType());
        updateById(reportTypeProjectTypeRel);
    }

    /**
     * 查询报告类型和项目所属功能类型关联关系
     *
     * @return List<ReportTypeProjectTypeRel>
     * @author lixing
     * @version V1.0 2021-05-19 11:20:23
     */
    @Override
    public List<ReportTypeProjectTypeRel> queryReportTypeProjectTypeRelList(QueryReportTypeProjectTypeRelDTO queryReportTypeProjectTypeRelDTO) {
        QueryWrapper<ReportTypeProjectTypeRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportTypeProjectTypeRel.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ReportTypeProjectTypeRel.PROP_CREATIONTIME);

        if (queryReportTypeProjectTypeRelDTO != null) {

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportTypeProjectTypeRelDTO.getConfigId())) {
                queryWrapper.like(ReportTypeProjectTypeRel.PROP_CONFIG_ID, queryReportTypeProjectTypeRelDTO.getConfigId());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportTypeProjectTypeRelDTO.getFunctionType())) {
                queryWrapper.like(ReportTypeProjectTypeRel.PROP_FUNCTION_TYPE, queryReportTypeProjectTypeRelDTO.getFunctionType());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询报告类型和项目所属功能类型关联关系
     *
     * @return IPage<ReportTypeProjectTypeRel>
     * @author lixing
     * @version V1.0 2021-05-19 11:20:23
     */
    @Override
    public IPage<ReportTypeProjectTypeRel> pageQueryReportTypeProjectTypeRel(PageQueryReportTypeProjectTypeRelDTO pageQueryReportTypeProjectTypeRelDTO) {
        QueryWrapper<ReportTypeProjectTypeRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportTypeProjectTypeRel.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ReportTypeProjectTypeRel> pageParam = new Page<>(pageQueryReportTypeProjectTypeRelDTO.getPage(), pageQueryReportTypeProjectTypeRelDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryReportTypeProjectTypeRelDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, ReportTypeProjectTypeRel.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryReportTypeProjectTypeRelDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportTypeProjectTypeRelDTO.getConfigId())) {
            queryWrapper.like(ReportTypeProjectTypeRel.PROP_CONFIG_ID, pageQueryReportTypeProjectTypeRelDTO.getConfigId());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportTypeProjectTypeRelDTO.getFunctionType())) {
            queryWrapper.like(ReportTypeProjectTypeRel.PROP_FUNCTION_TYPE, pageQueryReportTypeProjectTypeRelDTO.getFunctionType());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }
}
