package com.persagy.apm.energy.report.saasweb.client;

import com.persagy.apm.common.model.dto.PoemsFeignBaseDTO;
import com.persagy.apm.common.response.PoemsFeignResponse;
import com.persagy.apm.energy.report.common.FeignClientNameConstants;
import com.persagy.apm.energy.report.common.FeignConfig;
import com.persagy.apm.energy.report.saasweb.client.fallback.SaasWebClientFallbackFactory;
import com.persagy.apm.energy.report.saasweb.model.dto.QueryBuildsByProjectIdDTO;
import com.persagy.apm.energy.report.saasweb.model.vo.BuildingInfoVO;
import com.persagy.apm.energy.report.saasweb.model.vo.UserPartitionProjectsInfoVO;
import com.persagy.apm.energy.report.saasweb.model.vo.UserProjectsInfoVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;

/**
 * 运维平台feign客户端
 *
 * @author lixing
 * @version V1.0 2021/3/22 2:13 下午
 **/
@FeignClient(name = FeignClientNameConstants.SAAS_WEB, fallbackFactory = SaasWebClientFallbackFactory.class,
        configuration = FeignConfig.class)
public interface SaasWebClient {
    /**
     * 根据项目id，加载项目下的建筑信息
     *
     * @param queryBuildsByProjectIdDTO 查询条件
     * @return 项目下的建筑信息
     * @author lixing
     * @version V1.0 2021/5/19 2:27 下午
     */
    @PostMapping("/Spring/MVC/entrance/unifier/listBuildingsByProjectIdService")
    PoemsFeignResponse<BuildingInfoVO> listBuildingsByProjectId(QueryBuildsByProjectIdDTO queryBuildsByProjectIdDTO);

    /**
     * 获取用户信息（基本信息和关联的项目）
     *
     * @param poemsFeignBaseDTO 登录信息
     * @return 用户信息
     * @author lixing
     * @version V1.0 2021/5/19 3:36 下午
     */
    @PostMapping("/Spring/MVC/entrance/unifier/saasGetCurrentUserService")
    PoemsFeignResponse<UserProjectsInfoVO> getCurrentUserProjectsInfo(PoemsFeignBaseDTO poemsFeignBaseDTO);

    /**
     * 获取用户信息（基本信息和关联的分区项目树）
     *
     * @param poemsFeignBaseDTO 登录信息
     * @return 用户信息
     * @author lixing
     * @version V1.0 2021/5/19 3:36 下午
     */
    @PostMapping("/Spring/MVC/entrance/unifier/saasGetCurrentUserProjectsTreeService")
    PoemsFeignResponse<UserPartitionProjectsInfoVO> getCurrentUserPartitionProjectsInfo(PoemsFeignBaseDTO poemsFeignBaseDTO);
}
